# Generated by Django 2.2.11 on 2020-03-19 16:58

from django.db import migrations

from django_yubin import tasks


# To avoid dependeny with constants.py
RESULT_SENT = 0
RESULT_SKIPPED = 1
RESULT_FAILED = 2


def migrate_to_queues(apps, schema_editor):
    from django_yubin.models import Message as DBMessage

    Message = apps.get_model('django_yubin', 'Message')
    QueuedMessage = apps.get_model('django_yubin', 'QueuedMessage')
    Log = apps.get_model('django_yubin', 'Log')

    # Messages without a QueueMessage ara sent.
    for message in Message.objects.all():
        queued = QueuedMessage.objects.filter(message=message).only('date_queued').first()
        if queued:
            message.status = DBMessage.STATUS_QUEUED
            message.date_enqueued = queued.date_queued
            message.enqueued_count = 1
            message.save()
            tasks.send_email.delay(message.pk)
        else:
            message.status = DBMessage.STATUS_SENT
            message.sent_count = 1
            message.date_enqueued = message.date_sent
            message.enqueued_count = 1
            message.save()

    # Set Log actions based on its result
    for log in Log.objects.all():
        if log.result == RESULT_SENT:
            log.action = DBMessage.STATUS_SENT
        elif log.result == RESULT_FAILED:
            log.action = DBMessage.STATUS_FAILED
        elif log.result == RESULT_SKIPPED:
            log.action = DBMessage.STATUS_DISCARDED
        log.save()


class Migration(migrations.Migration):

    dependencies = [
        ('django_yubin', '0006_auto_20200319_1120'),
    ]

    operations = [
        migrations.RunPython(migrate_to_queues),
    ]



from celery import shared_task


@shared_task()
def send_email(message_pk):
    """
    Send an email from a database Message PK.
    """
    from .engine import send_db_message
    return send_db_message(message_pk)


@shared_task()
def retry_emails(max_retries=3):
    """
    Retry sending retryable emails enqueueing them again.
    """
    from .models import Message
    enqueued, failed = Message.retry_messages(max_retries)
    return enqueued, failed


@shared_task()
def delete_old_emails(days=90):
    """
    Delete emails created before `days` days (default 90).
    """
    from .models import Message
    deleted, cutoff_date = Message.delete_old(days)
    return deleted, cutoff_date



from django.core.management.base import BaseCommand, CommandError

from ...storage_backends import file2db, StorageBackendException


class Command(BaseCommand):
    help = 'Migrate emails from FileStorageBackend storage to DatabaseStorageBackend.'

    def add_arguments(self, parser):
        parser.add_argument(
            '--delete',
            action='store_true',
            help='Delete files from the FileStorageBackend.',
        )

    def handle(self, *args, **options):
        delete = options['delete']
        try:
            file2db(delete)
        except StorageBackendException as e:
            raise CommandError(e)



import datetime
import logging

from django.core.exceptions import FieldError
from django.core.mail.message import EmailMessage, EmailMultiAlternatives
from django.db import models
from django.db.models import F
from django.utils.module_loading import import_string
from django.utils.text import Truncator
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _
import mailparser

from . import mailparser_utils, tasks


logger = logging.getLogger(__name__)


class MessageQuerySet(models.QuerySet):
    def retryable(self, max_retries=0):
        qs = self.filter(status__gte=self.model.STATUS_FAILED)
        if max_retries > 0:
            qs = qs.filter(enqueued_count__lt=max_retries)
        return qs


class MessageManager(models.Manager):
    def get_queryset(self):
        return MessageQuerySet(self.model, using=self._db)

    def retryable(self, max_retries=0):
        return self.get_queryset().retryable(max_retries=max_retries)


class Message(models.Model):
    """
    An email message.
    """
    STATUS_CREATED = 0
    STATUS_QUEUED = 1
    STATUS_IN_PROCESS = 2
    STATUS_SENT = 3
    STATUS_FAILED = 4
    STATUS_BLACKLISTED = 5
    STATUS_DISCARDED = 6
    STATUS_CHOICES = (
        (STATUS_CREATED, _('Created')),
        (STATUS_QUEUED, _('Queued')),
        (STATUS_IN_PROCESS, _('In process')),
        (STATUS_SENT, _('Sent')),
        (STATUS_FAILED, _('Failed')),
        (STATUS_BLACKLISTED, _('Blacklisted')),
        (STATUS_DISCARDED, _('Discarded')),
    )

    to_address = models.TextField(_('to addresses'))
    cc_address = models.TextField(_('cc addresses'), blank=True, default="")
    bcc_address = models.TextField(_('bcc addresses'), blank=True, default="")

    # These fields are to easy access, filtering and searching without parsing the email.
    from_address = models.CharField(_('from address'), max_length=200)
    subject = models.CharField(_('subject'), max_length=255)

    # This field is for internal use in storage backends. They can use it to save the email
    # like the DatabaseSorageBackend, the file path like the FileStorageBackend, etc.
    # Other users must access this data through the ``message_data`` property.
    _message_data = models.TextField(_('message data'), db_column='message_data')

    @property
    def message_data(self):
        storage_backend = import_string(self.storage)
        return storage_backend.get_message_data(self)

    @message_data.setter
    def message_data(self, data):
        storage_backend = import_string(self.storage)
        storage_backend.set_message_data(self, data)

    storage = models.CharField(_('storage backend'), max_length=200, blank=False,
                               default="django_yubin.storage_backends.DatabaseStorageBackend")

    date_created = models.DateTimeField(_('date created'), auto_now_add=True)
    date_sent = models.DateTimeField(_('date sent'), null=True, blank=True)
    sent_count = models.PositiveSmallIntegerField(_('sent count'), default=0,
                                                  help_text=_('Times the message has been sent'))
    date_enqueued = models.DateTimeField(_('date enqueued'), null=True, blank=True)
    enqueued_count = models.PositiveSmallIntegerField(_('enqueued count'), default=0,
                                                      help_text=_('Times the message has been enqueued'))
    status = models.PositiveSmallIntegerField(choices=STATUS_CHOICES, default=STATUS_CREATED)

    objects = MessageManager()

    class Meta:
        ordering = ('date_created',)
        verbose_name = _('message')
        verbose_name_plural = _('messages')

    def __init__(self, *args, **kwargs):
        if '_message_data' in kwargs:
            raise FieldError("_message_data can not be used for creating instances, use message_data.")
        return super().__init__(*args, **kwargs)

    def __str__(self):
        recipients = self.to_address
        if self.cc_address:
            recipients += ', %s' % self.cc_address
        if self.bcc_address:
            recipients += ', %s' % self.bcc_address
        return '%s: %s' % (recipients, self.subject)

    def to(self):
        return [email.strip() for email in self.to_address.split(",") if email.strip()]

    def cc(self):
        return [email.strip() for email in self.cc_address.split(",") if email.strip()]

    def bcc(self):
        return [email.strip() for email in self.bcc_address.split(",") if email.strip()]

    def recipients(self):
        return self.to() + self.cc() + self.bcc()

    def get_message_parser(self):
        return mailparser.parse_from_string(self.message_data)

    def get_email_message(self):
        """
        Returns EmailMultiAlternatives or EmailMessage depending on whether the email is multipart or not.
        """
        msg = self.get_message_parser()
        to = self.to() or mailparser_utils.get_addresses(msg.to)
        cc = self.cc() or mailparser_utils.get_addresses(msg.cc)
        bcc = self.bcc() or mailparser_utils.get_addresses(msg.bcc)

        Email = EmailMultiAlternatives if msg.text_html else EmailMessage
        email = Email(
            subject=msg.subject,
            body='\n'.join(msg.text_plain),
            from_email=mailparser_utils.get_address(msg.from_),
            to=to, cc=cc, bcc=bcc,
        )

        if msg.text_html:
            email.attach_alternative('<br>'.join(msg.text_html), mimetype='text/html')

        for attachment in msg.attachments:
            email.attach(
                attachment['filename'],
                mailparser_utils.get_content(attachment),
                attachment['mail_content_type'],
            )

        return email

    def add_log(self, log_message):
        Log.objects.create(message=self, action=self.status, log_message=log_message)

    def mark_as(self, status, log_message=None):
        should_refresh_from_db = False

        self.status = status
        if status is self.STATUS_SENT:
            self.date_sent = now()
            self.sent_count = F('sent_count') + 1
            should_refresh_from_db = True
        elif status is self.STATUS_QUEUED:
            self.date_enqueued = now()
            self.enqueued_count = F('enqueued_count') + 1
            should_refresh_from_db = True
        self.save()

        if should_refresh_from_db:
            self.refresh_from_db()

        if log_message is not None:
            self.add_log(log_message)

    def can_be_enqueued(self):
        return self.status in (
            self.STATUS_CREATED,
            self.STATUS_FAILED,
            self.STATUS_BLACKLISTED,
            self.STATUS_DISCARDED
        )

    def enqueue(self, log_message=None):
        """
        Sends the task to enqueue itself taking care of undoing changes if the delivery fails.
        """
        if not self.can_be_enqueued():
            self.add_log("Message can not be enqueued in it's current status")
            return False

        backup = {
            'date_enqueued': self.date_enqueued,
            'enqueued_count': self.enqueued_count,
            'status': self.status,
        }
        self.mark_as(self.STATUS_QUEUED, log_message)

        try:
            tasks.send_email.delay(self.pk)
            return True
        except Exception as e:
            self.date_enqueued = backup['date_enqueued']
            self.enqueued_count = backup['enqueued_count']
            self.status = backup['status']
            self.save()
            self.add_log('Error enqueuing email: {}'.format(e))
        return False

    @classmethod
    def retry_messages(cls, max_retries=3):
        enqueued = 0
        messages = cls.objects.retryable(max_retries)
        for message in messages:
            enqueued += message.enqueue('Retry sending the email.')
        failed = len(messages) - enqueued
        return enqueued, failed

    @classmethod
    def delete_old(cls, days=90):
        """
        Deletes mails created before `days` days.

        Returns the deletion data from Django and the cuttoff date.
        """
        cutoff_date = now() - datetime.timedelta(days)
        deleted = cls.objects.filter(date_created__lt=cutoff_date).delete()
        return deleted, cutoff_date


class Blacklist(models.Model):
    """
    A blacklisted email address.

    Messages attempted to be sent to e-mail addresses which appear on this
    blacklist will be skipped entirely.
    """
    email = models.EmailField(_('email'), max_length=200)
    date_added = models.DateTimeField(_('date added'), default=now)

    class Meta:
        ordering = ('-date_added',)
        verbose_name = _('blacklisted email')
        verbose_name_plural = _('blacklisted emails')

    def __str__(self):
        return self.email


class Log(models.Model):
    """
    A log used to record the activity of a queued message.
    """
    message = models.ForeignKey(Message, verbose_name=_('message'), editable=False, on_delete=models.CASCADE)
    action = models.PositiveSmallIntegerField(_('action'), choices=Message.STATUS_CHOICES,
                                              default=Message.STATUS_CREATED)
    date = models.DateTimeField(_('date'), auto_now_add=True)
    log_message = models.TextField(_('log'), blank=True)

    class Meta:
        ordering = ('-date',)
        verbose_name = _('log')
        verbose_name_plural = _('logs')

    def __str__(self):
        return Truncator(self.log_message).chars(50)



import base64
import hashlib


def get_content(attachment):
    return base64.b64decode(attachment['payload'])


def get_signature(attachment):
    return hashlib.md5(get_content(attachment)).hexdigest()


def get_attachment(mailparser, signature):
    """
    Given a msg returns the attachment with the given signature.
    """
    for attachment in mailparser.attachments:
        if get_signature(attachment) == signature:
            return attachment
    return None


def get_address(mailparse_addresses):
    """
    Returns 'Foo Bar <foo.bar@baz.com>' or 'foo.bar@baz.com'.
    """
    addresses = get_addresses(mailparse_addresses)
    return addresses[0] if addresses else None


def get_addresses(mailparse_addresses):
    """
    Returns ['Foo Bar <foo.bar@baz.com>', ...] or ['foo.bar@baz.com', ...].
    """
    addresses = []
    for address_parts in mailparse_addresses:
        if not address_parts[0] or '@' in address_parts[0]:
            addresses.append(address_parts[1])
        else:
            addresses.append('{} <{}>'.format(address_parts[0], address_parts[1]))
    return addresses



# Generated by Django 2.2.11 on 2020-03-20 09:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_yubin', '0007_auto_20200319_1158'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='log',
            name='result',
        ),
        migrations.DeleteModel(
            name='QueuedMessage',
        ),
    ]



# Generated by Django 3.2.16 on 2022-11-30 10:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_yubin', '0009_auto_20221122_1605'),
    ]

    operations = [
        migrations.AddField(
            model_name='message',
            name='storage',
            field=models.CharField(default='django_yubin.storage_backends.DatabaseStorageBackend', max_length=200, verbose_name='storage backend'),
        ),
    ]



"""
High level functions to queue emails.
"""

import logging


logger = logging.getLogger(__name__)


def queue_email_message(email_message, fail_silently=False):
    """
    Add new messages to the email queue.

    The ``email_message`` argument should be an instance of Django's core mail
    ``EmailMessage`` class.

    The ``fail_silently`` argument is not used and is only provided to match
    the signature of the ``EmailMessage.send`` function which it may emulate.
    """
    from . import models, settings

    if settings.MAILER_TEST_MODE and settings.MAILER_TEST_EMAIL:
        email_message = _set_message_test_mode(email_message, settings.MAILER_TEST_EMAIL)

    if not email_message.recipients():
        return 0

    message = models.Message.objects.create(
        to_address=','.join(email_message.to),
        cc_address=','.join(email_message.cc),
        bcc_address=','.join(email_message.bcc),
        from_address=email_message.from_email,
        subject=email_message.subject,
        message_data=email_message.message().as_string(),
        storage=settings.MAILER_STORAGE_BACKEND)

    if message.enqueue('Enqueued from a Backend or django-yubin itself.'):
        return 1
    else:
        logger.exception('Error enqueuing an email', extra={'email_message': message})
        return 0


def _set_message_test_mode(email_message, mailer_test_email):
    """
    Sets the headers of the message with test values when
    ``MAILER_TEST_MODE`` setting is ``True``
    """
    original_to = ','.join(email_message.to)
    email_message.extra_headers['X-Yubin-Test-Original'] = original_to
    email_message.to = [mailer_test_email]
    email_message.cc = []
    email_message.bcc = []
    return email_message


def send_mail(subject, message, from_email, recipient_list,
              fail_silently=False, auth_user=None, auth_password=None):
    """
    Add a new message to the mail queue.

    This is a replacement for Django's ``send_mail`` core email method.

    The `fail_silently``, ``auth_user`` and ``auth_password`` arguments are
    only provided to match the signature of the emulated function. These
    arguments are not used.
    """
    from django.core.mail import EmailMessage
    from django.utils.encoding import force_str

    subject = force_str(subject)
    email_message = EmailMessage(subject, message, from_email, recipient_list)
    queue_email_message(email_message)


def mail_admins(subject, message, fail_silently=False):
    """
    Add one or more new messages to the mail queue addressed to the site
    administrators (defined in ``settings.ADMINS``).

    This is a replacement for Django's ``mail_admins`` core email method.

    The ``fail_silently`` argument is only provided to match the signature of
    the emulated function. This argument is not used.
    """
    from django.conf import settings as django_settings
    from django.utils.encoding import force_str

    subject = django_settings.EMAIL_SUBJECT_PREFIX + force_str(subject)
    from_email = django_settings.SERVER_EMAIL
    recipient_list = [recipient[1] for recipient in django_settings.ADMINS]
    send_mail(subject, message, from_email, recipient_list)


def mail_managers(subject, message, fail_silently=False):
    """
    Add one or more new messages to the mail queue addressed to the site
    managers (defined in ``settings.MANAGERS``).

    This is a replacement for Django's ``mail_managers`` core email method.

    The ``fail_silently`` argument is only provided to match the signature of
    the emulated function. This argument is not used.
    """
    from django.conf import settings as django_settings
    from django.utils.encoding import force_str

    subject = django_settings.EMAIL_SUBJECT_PREFIX + force_str(subject)
    from_email = django_settings.SERVER_EMAIL
    recipient_list = [recipient[1] for recipient in django_settings.MANAGERS]
    send_mail(subject, message, from_email, recipient_list)



# Generated by Django 2.2.11 on 2020-03-19 16:20

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('django_yubin', '0005_auto_20181128_0407'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='blacklist',
            options={'ordering': ('-date_added',), 'verbose_name': 'blacklisted email', 'verbose_name_plural': 'blacklisted emails'},
        ),
        migrations.AlterModelOptions(
            name='log',
            options={'ordering': ('-date',), 'verbose_name': 'log', 'verbose_name_plural': 'logs'},
        ),
        migrations.AlterModelOptions(
            name='message',
            options={'ordering': ('date_created',), 'verbose_name': 'message', 'verbose_name_plural': 'messages'},
        ),
        migrations.AddField(
            model_name='log',
            name='action',
            field=models.PositiveSmallIntegerField(choices=[(0, 'Created'), (1, 'Queued'), (2, 'In process'), (3, 'Sent'), (4, 'Failed'), (5, 'Blacklisted'), (6, 'Discarded')], default=0, verbose_name='action'),
        ),
        migrations.AddField(
            model_name='message',
            name='date_enqueued',
            field=models.DateTimeField(blank=True, null=True, verbose_name='date enqueued'),
        ),
        migrations.AddField(
            model_name='message',
            name='enqueued_count',
            field=models.PositiveSmallIntegerField(default=0, help_text='Times the message has been enqueued', verbose_name='enqueued count'),
        ),
        migrations.AddField(
            model_name='message',
            name='sent_count',
            field=models.PositiveSmallIntegerField(default=0, help_text='Times the message has been sent', verbose_name='sent count'),
        ),
        migrations.AddField(
            model_name='message',
            name='status',
            field=models.PositiveSmallIntegerField(choices=[(0, 'Created'), (1, 'Queued'), (2, 'In process'), (3, 'Sent'), (4, 'Failed'), (5, 'Blacklisted'), (6, 'Discarded')], default=0),
        ),
        migrations.AlterField(
            model_name='blacklist',
            name='date_added',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='date added'),
        ),
        migrations.AlterField(
            model_name='blacklist',
            name='email',
            field=models.EmailField(max_length=200, verbose_name='email'),
        ),
        migrations.AlterField(
            model_name='log',
            name='date',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='date'),
        ),
        migrations.AlterField(
            model_name='log',
            name='log_message',
            field=models.TextField(blank=True, verbose_name='log'),
        ),
        migrations.AlterField(
            model_name='log',
            name='message',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, to='django_yubin.Message', verbose_name='message'),
        ),
        migrations.AlterField(
            model_name='log',
            name='result',
            field=models.PositiveSmallIntegerField(choices=[(0, 'success'), (1, 'not sent (blacklisted or paused)'), (2, 'failure')], help_text='Deprecated for new emails, see "action"', null=True),
        ),
        migrations.AlterField(
            model_name='message',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True, verbose_name='date created'),
        ),
        migrations.AlterField(
            model_name='message',
            name='date_sent',
            field=models.DateTimeField(blank=True, null=True, verbose_name='date sent'),
        ),
        migrations.AlterField(
            model_name='message',
            name='encoded_message',
            field=models.TextField(verbose_name='encoded message'),
        ),
        migrations.AlterField(
            model_name='message',
            name='from_address',
            field=models.CharField(max_length=200, verbose_name='from address'),
        ),
        migrations.AlterField(
            model_name='message',
            name='subject',
            field=models.CharField(max_length=255, verbose_name='subject'),
        ),
        migrations.AlterField(
            model_name='message',
            name='to_address',
            field=models.CharField(max_length=200, verbose_name='to address'),
        ),
    ]



from django.contrib import admin, messages as dj_messages
from django.http import HttpResponse
from django.shortcuts import render
from django.urls import re_path, reverse
from django.utils.module_loading import import_string
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from django.views.decorators.clickjacking import xframe_options_sameorigin

from . import mailparser_utils, models, settings


class LogInline(admin.TabularInline):
    model = models.Log
    readonly_fields = ('action', 'date', 'log_message')

    def has_add_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


@admin.register(models.Message)
class MessageAdmin(admin.ModelAdmin):

    @admin.display(description=_('Show'))
    def message_link(self, instance):
        url = reverse('admin:mail_detail', args=(instance.id,))
        return mark_safe(f'<a href="{url}" onclick="return showAddAnotherPopup(this);">Show</a>')

    @admin.display(description=_('Message data'))
    def message_data(self, instance):
        backend = import_string(settings.MAILER_STORAGE_BACKEND)
        return mark_safe(backend.admin_display_message_data(self, instance))

    @admin.display(description=_('Storage'))
    def storage_class(self, instance):
        return mark_safe(instance.storage.split('.')[-1])

    list_display = ('from_address', 'to_address', 'subject', 'date_created', 'date_sent',
                    'date_enqueued', 'status', 'storage_class', 'message_link')
    list_filter = ('date_created', 'date_sent', 'date_enqueued', 'status')
    fields = ('from_address', 'to_address', 'cc_address', 'bcc_address', 'subject', 'message_data',
              'storage', 'date_sent', 'sent_count', 'date_enqueued', 'enqueued_count', 'status')
    readonly_fields = ('to_address', 'cc_address', 'bcc_address', 'from_address', 'subject', 'message_data',
                       'storage', 'date_created')
    search_fields = ('to_address', 'subject', 'from_address')
    date_hierarchy = 'date_created'
    ordering = ('-date_created',)
    actions = ['enqueue_action', 'mark_as_sent_action', 'mark_as_created_action']
    inlines = [LogInline]

    def enqueue_action(self, request, queryset):
        failed, queued = [], []
        for message in queryset:
            if message.enqueue('Enqueued from the admin.'):
                queued.append(str(message.pk))
            else:
                failed.append(str(message.pk))

        msg = _("{q_count} emails enqueued: {q} | {f_count} emails failed: {f}".format(
                    q_count=len(queued), q=','.join(queued),
                    f_count=len(failed), f=','.join(failed),
                ))
        if failed and queued:
            level = dj_messages.WARNING
        elif failed:
            level = dj_messages.ERROR
        elif queued:
            level = dj_messages.SUCCESS
        else:
            level = dj_messages.INFO

        self.message_user(request, msg, level)
    enqueue_action.short_description = _('Enqueue selected messages')

    def mark_as_sent_action(self, request, queryset):
        for message in queryset:
            message.mark_as(models.Message.STATUS_SENT, 'Marked as sent from the admin.')
        self.message_user(request, _("Emails marked as sent."), level=dj_messages.SUCCESS)
    mark_as_sent_action.short_description = _('Mark as sent selected messages')

    def mark_as_created_action(self, request, queryset):
        for message in queryset:
            message.mark_as(models.Message.STATUS_CREATED, 'Marked as created from the admin.')
        self.message_user(request, _("Emails marked as created."), level=dj_messages.SUCCESS)
    mark_as_created_action.short_description = _('Mark as created selected messages')

    def get_urls(self):
        urls = super(MessageAdmin, self).get_urls()
        custom_urls = [
            re_path(r'^mail/(?P<pk>\d+)/$',
                    self.admin_site.admin_view(self.detail_view),
                    name='mail_detail'),
            re_path(r'^mail/attachment/(?P<pk>\d+)/(?P<signature>[0-9a-f]{32})/$',
                    self.admin_site.admin_view(self.download_view),
                    name="mail_download"),
            re_path(r'^mail/html/(?P<pk>\d+)/$',
                    self.admin_site.admin_view(self.html_view),
                    name="mail_html"),
        ]
        return custom_urls + urls

    def detail_view(self, request, pk):
        instance = models.Message.objects.get(pk=pk)
        msg = instance.get_message_parser()
        context = {
            "subject": msg.subject,
            "from": mailparser_utils.get_address(msg.from_),
            "to": mailparser_utils.get_addresses(msg.to),
            "cc": mailparser_utils.get_addresses(msg.cc),
            "msg_text": "\n".join(msg.text_plain),
            "msg_html": "</br>".join(msg.text_html),
            "attachments": [
                {
                    'filename': attachment['filename'],
                    'content_type': attachment['mail_content_type'],
                    'size': len(mailparser_utils.get_content(attachment)),
                    'signature': mailparser_utils.get_signature(attachment),
                }
                for attachment in msg.attachments
            ],
            "is_popup": True,
            "object": instance,
        }
        return render(request, "django_yubin/message_detail.html", context)

    def download_view(self, request, pk, signature):
        instance = models.Message.objects.get(pk=pk)
        msg = instance.get_message_parser()
        attachment = mailparser_utils.get_attachment(msg, signature)
        response = HttpResponse(content_type=attachment['mail_content_type'])
        response['Content-Disposition'] = attachment['content-disposition']
        response.write(mailparser_utils.get_content(attachment))
        return response

    @xframe_options_sameorigin
    def html_view(self, request, pk):
        instance = models.Message.objects.get(pk=pk)
        msg = instance.get_message_parser()
        context = {"msg_html": "</br>".join(msg.text_html)}
        return render(request, "django_yubin/html_detail.html", context)


@admin.register(models.Blacklist)
class BlacklistAdmin(admin.ModelAdmin):
    list_display = ('email', 'date_added')


@admin.register(models.Log)
class LogAdmin(admin.ModelAdmin):
    def message_link(self, obj):
        url = reverse('admin:mail_detail', args=(obj.message.id,))
        return mark_safe("""<a href="%s" onclick="return showAddAnotherPopup(this);">show</a>""" % url)
    message_link.allow_tags = True
    message_link.short_description = _('Message')

    def message__to_address(self, obj):
        return obj.message.to_address
    message__to_address.admin_order_field = 'message__to_address'

    def message__subject(self, obj):
        return obj.message.subject
    message__subject.admin_order_field = 'message__subject'

    list_select_related = True
    list_display = ('id', 'date', 'action', 'log_message', 'message__to_address', 'message__subject', 'message_link')
    list_filter = ('action', 'date')
    list_display_links = ('id',)
    date_hierarchy = 'date'



"""
Email backends.
"""

from django.core.mail.backends.base import BaseEmailBackend

from django_yubin import queue_email_message


class QueuedEmailBackend(BaseEmailBackend):
    """
    A wrapper that manages a queued SMTP system.
    """

    def send_messages(self, email_messages):
        """
        Add new messages to the email queue.

        The ``email_messages`` argument should be one or more instances
        of Django's core mail ``EmailMessage`` class.
        """
        num_sent = 0
        for email_message in email_messages:
            num_sent += queue_email_message(email_message)
        return num_sent



from django.http import HttpResponse
from django.utils import timezone
from django.views.generic import View

from . import settings
from .models import Message


class MailHealthCheckView(View):
    def get(self, request, *args, **kwargs):
        try:
            threshold = int(request.GET.get('t', settings.MAILER_HC_QUEUED_LIMIT_OLD))
        except ValueError:
            return HttpResponse("Threshold error, it should be an integer",
                                content_type='text/plain', status=400)
        oldest, too_old = 0, False

        message = Message.objects.filter(status__lt=Message.STATUS_SENT) \
                                 .order_by('date_created').first()
        if message:
            seconds = (timezone.now().astimezone() - message.date_created.astimezone()).total_seconds()
            oldest = round(seconds / 60)  # to minutes
            too_old = oldest > threshold

        response = '\n'.join((
            'oldest_queued_email: {} mins'.format(oldest),
            'emails_queued_too_old: {}'.format(('no', 'yes')[too_old]),
            'threshold: {} mins'.format(threshold),
        ))
        return HttpResponse(response, content_type='text/plain', status=(200, 500)[too_old])



#!/usr/bin/env python3

from setuptools import setup


INSTALL_REQUIRES = [
    'celery>=5.0,<5.3',
    'mail-parser',
    'pytz',
]


with open('README.rst') as docs_index:
    long_description = docs_index.read()


def get_version():
    version_globals = {}
    with open('django_yubin/version.py') as f:
        exec(f.read(), version_globals)
    return version_globals['__version__']


setup(
    name='django-yubin',
    version=get_version(),
    description=("A reusable Django app for composing and queueing emails "
                 "django-mailviews + Celery + others"),
    long_description=long_description,
    author='Antoni Aloy',
    author_email='aaloy@apsl.net',
    maintainer='APSL',
    maintainer_email='info@apsl.net',
    url='http://github.com/APSL/django-yubin',
    install_requires=INSTALL_REQUIRES,
    packages=[
        'django_yubin',
        'django_yubin.management',
        'django_yubin.management.commands',
        'django_yubin.migrations',
    ],
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
        'Framework :: Django :: 4.1',
        'Framework :: Django :: 4.2',
    ]
)



# Generated by Django 3.2.16 on 2022-11-22 16:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_yubin', '0008_auto_20200320_0407'),
    ]

    operations = [
        migrations.AlterField(
            model_name='log',
            name='date',
            field=models.DateTimeField(auto_now_add=True, verbose_name='date'),
        ),
        migrations.RenameField(
            model_name='message',
            old_name='encoded_message',
            new_name='_message_data',
        ),
        migrations.AlterField(
            model_name='message',
            name='_message_data',
            field=models.TextField(db_column='message_data', verbose_name='message data'),
        ),
    ]
